#include <gtk/gtk.h>
#include <gtk/gtkmagic.h>

typedef struct _GtkBorderwidthPanel GtkBorderwidthPanel;

struct _GtkBorderwidthPanel
{
	GtkMagic *magic;
	GtkProp *prop;
	GtkWidget *from_parent;
	GtkAdjustment *adjustment;
	GtkWidget *sensitive_container;
};

void
gtk_borderwidth_panel_prop_data_changed(GtkProp *prop, GPropData *data_new, gpointer user_data)
{
	GtkBorderwidthPanel *panel;
	
	panel = user_data;
	
	g_return_if_fail(data_new->type == G_PROP_DATA_UINT8);

	gtk_signal_handler_block_by_data(GTK_OBJECT(panel->adjustment), panel);
	gtk_adjustment_set_value(panel->adjustment, (gfloat)data_new->d.uint8_data);
	gtk_signal_handler_unblock_by_data(GTK_OBJECT(panel->adjustment), panel);
}

void
gtk_borderwidth_panel_prop_ownership_changed(GtkProp *prop, gboolean ownership_new, gpointer user_data)
{
	GtkBorderwidthPanel *panel;
	
	panel = user_data;
	
	gtk_widget_set_sensitive(panel->sensitive_container, ownership_new);
	gtk_signal_handler_block_by_data(GTK_OBJECT(panel->from_parent), panel);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(panel->from_parent), (!ownership_new));
	gtk_signal_handler_unblock_by_data(GTK_OBJECT(panel->from_parent), panel);
}

void
gtk_borderwidth_panel_slider_changed(GtkAdjustment *adj, gpointer user_data)
{
	GtkBorderwidthPanel *panel;
	GPropData *data;
	panel = user_data;
	
	data = g_prop_data_uint8_new((guint8)adj->value);
	
	gtk_magic_set_prop(panel->magic, panel->prop->name, data);
}

void
gtk_borderwidth_panel_from_parent_toggled(GtkToggleButton *button, gpointer user_data)
{
	GtkBorderwidthPanel *panel;
	gboolean active;

	panel = user_data;
	
	active = gtk_toggle_button_get_active(button);
	
	if (active)
	{
		gtk_magic_delete_prop(panel->magic, panel->prop->name);
	}
	else
	{
		gtk_magic_set_prop(panel->magic, panel->prop->name, panel->prop->data);
	}
}

void
gtk_borderwidth_panel_destroy(GtkWidget *widget, gpointer user_data)
{
	GtkBorderwidthPanel *panel;
	panel = user_data;
	
	gtk_magic_unreg_prop(panel->magic, panel->prop);
	gtk_object_destroy(GTK_OBJECT(panel->prop));
	
	g_free(panel);
}

GtkWidget*
gtk_borderwidth_panel_new(GtkMagic *magic, gchar *widget_name)
{
	GtkObject *adj;
	GtkWidget *frame, *vbox, *from_parent,*hscale;
	GtkProp *prop;
	gchar *prop_name;
	GtkBorderwidthPanel *panel;
	
	prop_name = g_strconcat(widget_name,"::borderwidth", NULL);
	
	prop = gtk_magic_observe_prop(magic, prop_name, GMAGIC_OBSERVE_DATA|GMAGIC_OBSERVE_OWNERSHIP);
	if (prop == NULL)
	{
		g_print("couldn't observe!\n");
		return NULL;
	}
	
	g_return_val_if_fail(prop->data->type == G_PROP_DATA_UINT8, NULL);
	
	panel = g_new(GtkBorderwidthPanel, 1);
	
	panel->magic = magic;
	panel->prop = prop;
	

	frame = gtk_frame_new("border width");
	
		vbox = gtk_vbox_new(0,0);
		
		from_parent = gtk_check_button_new_with_label("from parent");
		gtk_widget_show(from_parent);
		gtk_box_pack_start(GTK_BOX(vbox), from_parent, 0,0,0);
		
		panel->from_parent = from_parent;

		adj = gtk_adjustment_new((gfloat)prop->data->d.uint8_data,
			   0.0,
			   20.0,
			   1.0,
			   5.0,
			   5.0);
		
		panel->adjustment = GTK_ADJUSTMENT(adj);
			
		gtk_signal_connect(GTK_OBJECT(adj),"value_changed", gtk_borderwidth_panel_slider_changed, panel);
		
		hscale = gtk_hscale_new(panel->adjustment);
		gtk_scale_set_draw_value(GTK_SCALE(hscale), TRUE),
		gtk_scale_set_digits(GTK_SCALE(hscale), 0);
		gtk_scale_set_value_pos(GTK_SCALE(hscale), GTK_POS_RIGHT);
		gtk_widget_show(hscale);
		
		panel->sensitive_container = hscale;

		gtk_box_pack_start(GTK_BOX(vbox), hscale, 0,0,0);
		
		gtk_widget_show(vbox);
	
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	
	if (!prop->ownership)
	{
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(from_parent), TRUE);
		gtk_widget_set_sensitive(panel->sensitive_container, FALSE);
	}

	gtk_signal_connect(GTK_OBJECT(from_parent), "toggled", gtk_borderwidth_panel_from_parent_toggled, panel);
	gtk_signal_connect(GTK_OBJECT(prop), "data_changed", gtk_borderwidth_panel_prop_data_changed, panel);
	gtk_signal_connect(GTK_OBJECT(prop), "ownership_changed", gtk_borderwidth_panel_prop_ownership_changed, panel);

	gtk_signal_connect(GTK_OBJECT(vbox), "destroy", gtk_borderwidth_panel_destroy, panel);
	
	return frame;
}
