#include "gtkconfig.h"
#include <gtk/gtk.h>

#include <sys/types.h>
#include <grp.h>
#include <pwd.h>
#include <unistd.h>
#include <string.h>

#include "style.h"
#include "borderwidth.h"
#include "shadow.h"

typedef enum
{
	PANEL_GTK_MAGIC = 0, 
	PANEL_RCSTYLE,
	PANEL_BORDERWIDTH,
	PANEL_SHADOW,
	PANEL_LAST
} PanelType;

char *panel_name[PANEL_LAST] =
{
	"gtkmagic",
	"rcstyle",
	"borderwidth",
	"shadow"
};

/* --- panel funcs --- */
void
clear_panel(GtkMagicConfig *config)
{
	if (config->focus_panel == NULL)
		return;

	gtk_widget_destroy(config->focus_panel);
	config->focus_panel = NULL;
}

void
set_panel(GtkMagicConfig *config, GtkWidget *widget)
{
	if (config->focus_panel == widget)
		return;
	
	gtk_container_add(GTK_CONTAINER(config->panel_container), widget);
	gtk_widget_show(widget);
	config->focus_panel = widget;
}

void
preset_changed (GtkMagicConfig *config, const gchar *in_preset_name)
{
	clear_panel(config);
	config->config_panel = gtk_magic_config_panel_new(config);
	set_panel(config, config->config_panel);
	gtk_magic_config_set_preset(config, in_preset_name);
}

/* --- gui callbacks --- */

void
preset_entry_activated (GtkEditable *editable, gpointer user_data)
{
	GtkMagicConfig *config;
	gchar *text;
	
	config = user_data;
	text = gtk_entry_get_text(GTK_ENTRY(editable));

		
	if ((config->preset_name != NULL) && (strcmp(text, config->preset_name) == 0))
		return;
	
	preset_changed(config, text);
};

void
preset_popup_hides (GtkWidget *widget, gpointer user_data)
{
	GtkMagicConfig *config;
	gchar *text;
	
	config = user_data;
	text = gtk_entry_get_text(config->preset_entry);
	
	if ((config->preset_name != NULL) && (strcmp(text, config->preset_name) == 0))
		return;
	
	preset_changed(config, text);
}

void
object_list_select_child (GtkList *list, GtkWidget *widget, gpointer userdata)
{
	GtkMagicConfig *config;
	gchar *widget_name;
	guint panel_flags;
	
	config = userdata;

	if (config->focus_item == GTK_LIST_ITEM(widget))
		return;
	config->focus_item = GTK_LIST_ITEM(widget);
	
	panel_flags = GPOINTER_TO_UINT(gtk_object_get_data(GTK_OBJECT(widget), "panel_flags"));
	widget_name = gtk_object_get_data(GTK_OBJECT(widget), "widget_name");
	
	if (panel_flags == PANEL_GTK_MAGIC)
	{
		clear_panel(config);
		config->config_panel = gtk_magic_config_panel_new(config);
		set_panel(config, config->config_panel);
	}
	else
	{	
		GtkWidget *child, *label;
		GtkWidget *nb;
		
		clear_panel(config);
		nb = gtk_notebook_new();
		if (panel_flags & (1<<PANEL_RCSTYLE))
		{
			child = gtk_style_panel_new(config->magic, widget_name);
			label = gtk_label_new("Style");
			gtk_widget_show(child);
			gtk_widget_show(label);
			gtk_notebook_append_page(GTK_NOTEBOOK(nb), child, label);
		}
		
		if (panel_flags & (1<<PANEL_BORDERWIDTH))
		{
			GtkWidget *align, *vbox, *sub;
			align = gtk_alignment_new(0.5,0.5,0.75,0.75);
			vbox = gtk_vbox_new(1,0);
	
			sub = gtk_borderwidth_panel_new(config->magic, widget_name);
			gtk_widget_show(sub);
			gtk_box_pack_start(GTK_BOX(vbox),sub,0,0,0);
					
			if (panel_flags & (1<<PANEL_SHADOW))
			{
	
				sub = gtk_shadow_panel_new(config->magic, widget_name);
				gtk_box_pack_start(GTK_BOX(vbox),sub,0,0,0);
				gtk_widget_show(sub);
				
			}	

			label = gtk_label_new("Border");
			gtk_container_add(GTK_CONTAINER(align), vbox);
			gtk_widget_show(vbox);
			gtk_widget_show(align);
			gtk_widget_show(label);
			gtk_notebook_append_page(GTK_NOTEBOOK(nb), align, label);
		}
		set_panel(config, nb);
	}
};

/* constructor */

GtkMagicConfig*
gtk_magic_config_new(gchar *appname)
{
	GtkMagicConfig *config;
	
	GtkWidget *vbox, *hbox, *t, *config_item, *list;
	
	GList *usercontext_l = NULL, *appcontext_l = NULL;
	GList *tmp1, *tmp2;
	GList *context_l = NULL;
	GList *item_l = NULL;
	
	uid_t uid;
	gid_t gid;
	struct passwd *passwd;
	struct group *group;
	
	
	config = g_new(GtkMagicConfig,1);
	
	/* build up combo box choose list */
	
	uid = getuid(); 
	passwd = getpwuid(uid);
	config->username = g_strconcat("user ",passwd->pw_name, NULL);
	
	gid = getgid();
	group = getgrgid(gid);
	config->groupname = g_strconcat("group ",group->gr_name, NULL);	

	usercontext_l = g_list_append(usercontext_l,config->username);
	usercontext_l = g_list_append(usercontext_l,config->groupname);
	usercontext_l = g_list_append(usercontext_l,"system");

	appcontext_l = g_list_append(appcontext_l,appname);
	appcontext_l = g_list_append(appcontext_l,"gtk+");
	
	tmp1 = usercontext_l;
	while(tmp1)
	{
		gchar *usercontext = tmp1->data;
		tmp2 = appcontext_l;
		while(tmp2)
		{
			gchar *name;
			gchar *appcontext = tmp2->data;
			name = g_strconcat(usercontext,"/",appcontext,NULL);
			context_l = g_list_append(context_l, name);
			tmp2 = tmp2->next;
		}
		tmp1 = tmp1->next;
	}
		
	/* gui */
	
	vbox = gtk_vbox_new(0,0);
	
		hbox = gtk_hbox_new(0,0);
	
			t = gtk_label_new("Preset:");
			gtk_widget_show(t);
			gtk_box_pack_start(GTK_BOX(hbox), t, 0,0,0);
	
			t = gtk_combo_new();
			gtk_combo_set_use_arrows(GTK_COMBO(t), 1);
			gtk_combo_disable_activate(GTK_COMBO(t));
			gtk_combo_set_popdown_strings(GTK_COMBO(t), context_l);
			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(t)->entry), "activate", preset_entry_activated, config);
			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(t)->popwin),"hide", preset_popup_hides, config);
			gtk_widget_show(t);
			gtk_box_pack_start(GTK_BOX(hbox), t, 1,1,0);
			
			config->preset_entry = GTK_ENTRY((GTK_COMBO(t)->entry));
			
		gtk_widget_show(hbox);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, 0,1,0);
		
		hbox = gtk_hbox_new(0,0);
		
			/* LIST */
		
			t = gtk_scrolled_window_new(NULL,NULL);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(t), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
				
			list = gtk_list_new();
			gtk_signal_connect(GTK_OBJECT(list),"select-child", object_list_select_child, config);

				/* GTK Magic Config Item */
				
				config_item = gtk_list_item_new_with_label("GtkMagicConfig");
				gtk_object_set_data(GTK_OBJECT(config_item),"panel_type", GUINT_TO_POINTER(PANEL_GTK_MAGIC));
				gtk_widget_show(config_item);
				item_l = g_list_append(item_l, config_item);
		
			gtk_list_append_items(GTK_LIST(list),item_l);
			gtk_widget_show(list);
			
			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(t), list);
			gtk_widget_show(t);
			
			gtk_box_pack_start(GTK_BOX(hbox), t, 1,1,0);

			config->list = GTK_LIST(list);
			config->focus_item = NULL;
				
			/* (empty panel = NULL) */
			
			t = gtk_alignment_new(0.5,0.5,0.9,0.9);
			gtk_widget_show(t);
			gtk_box_pack_start(GTK_BOX(hbox), t, 1,1,0);
				
			config->panel_container = t;
			config->focus_panel = NULL; 
		
		gtk_widget_show(hbox);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, 1,1,0);
	
	/* startup settings */	

	gtk_widget_show(vbox);
	
	gtk_list_item_select(GTK_LIST_ITEM(config_item));

	/* will initialise config_panel, server_addr_entry... */
	
	config->widget = vbox;
	
	/* set up other things */

	config->magic_server_addr = g_strdup(GMAGIC_DEFAULT_SERVER_ADDR);
	config->magic_info = g_strdup("GTK+ Magic Config");
	config->magic_usercontext_l = NULL;
	config->magic_appcontext_l = NULL;
	config->magic = NULL;
	
	config->preset_name = NULL;
	
	object_list_select_child(GTK_LIST(list),config_item, config);
	
	gtk_magic_config_set_preset(config, context_l->data);
	
	return config;
}

void
gtk_magic_config_server_config_changed(GtkMagicConfig *config)
{

	gint list_compare(gconstpointer a, gconstpointer b)
	{
		const GtkListItem *item_a, *item_b;
		gchar *name_a, *name_b;
		item_a = a;
		item_b = b;
		name_a = gtk_object_get_data(GTK_OBJECT(item_a),"widget_name");
		name_b = gtk_object_get_data(GTK_OBJECT(item_b),"widget_name");
		
		return strcmp(name_a, name_b);
	}
	gboolean create_item_list(gpointer key, gpointer data, gpointer user_data)
	{
		GList **list_p;
		gchar *widget_name;
		GList *item_l;
		GtkWidget *item;
		guint flags;
		
		list_p = user_data;
		
		item_l = *list_p;
		widget_name = key;
		flags = GPOINTER_TO_UINT(data);
		
		item = gtk_list_item_new_with_label(widget_name);
		gtk_object_set_data(GTK_OBJECT(item),"widget_name", g_strdup(widget_name));
		gtk_object_set_data(GTK_OBJECT(item),"panel_flags", GUINT_TO_POINTER(flags));
		gtk_widget_show(item);
		item_l = g_list_append(item_l, item);
		
		*list_p = item_l;
		
		return 1;
	}
	
	GtkMagic *magic;
	gboolean succ;
	
	magic = gtk_magic_new(config->magic_server_addr, config->magic_info, config->magic_usercontext_l, config->magic_appcontext_l);
	succ = gtk_magic_connect(magic);
	
	if (!succ)
	{
		g_warning("can't connect!\n");
		gtk_magic_destroy(magic);
		return;
	}
	else
	{
		GHashTable *ht;
		GList *list;
		
		GList *item_l = NULL;
		guint n;

		if (config->magic != NULL)	
			gtk_magic_destroy(config->magic);
		
		list = gtk_magic_get_names(magic, "Gtk");
			
		n = g_list_length(config->list->children);
		if (n > 1)
		{
			gtk_list_clear_items(config->list, 1, n);
		}
		
		ht = g_hash_table_new(g_str_hash, g_str_equal);
		
		while(list)
		{
			gchar **array;
			gchar *complete_propname;
			gchar *widget_name, *prop_name;
			guint i;
			
			complete_propname = list->data;
			array = g_strsplit(complete_propname,"::",1);
			
			widget_name = array[0];
			prop_name = array[1];

			for(i=0;i<PANEL_LAST;i++)
			{
				if (strcmp(panel_name[i], prop_name) == 0)
					break;
			}

			if (i != PANEL_LAST)
			{
				guint panel_flags;
				panel_flags = GPOINTER_TO_UINT(g_hash_table_lookup(ht, widget_name));
				panel_flags |= 1<<i;
				g_hash_table_insert(ht, widget_name, GUINT_TO_POINTER(panel_flags));
			}
			list=list->next;
		}
		
		g_hash_table_foreach_remove(ht, create_item_list, &item_l);
		g_hash_table_destroy(ht);
			
/*
			if (i != PANEL_LAST)
			{
				item = gtk_list_item_new_with_label(widget_name);
				gtk_object_set_data(GTK_OBJECT(item),"panel_type", GUINT_TO_POINTER(i));
				gtk_object_set_data(GTK_OBJECT(item),"prop_name", g_strdup(complete_propname));
				gtk_object_set_data(GTK_OBJECT(item),"widget_name", g_strdup(widget_name));
				gtk_widget_show(item);
				item_l = g_list_append(item_l, item);
			}	
			list=list->next;
		}
*/
		item_l = g_list_sort(item_l, list_compare);
		gtk_list_append_items(config->list, item_l);

		config->magic = magic;
	}
}

void
gtk_magic_config_set_preset (GtkMagicConfig *config, const gchar *in_preset_name)
{	      
	gchar *preset_name;
	gchar **array;
	gchar *user_target,
	      *app_target;
	GList *usercontext_l = NULL,
	      *appcontext_l = NULL;

	if (config->preset_name != NULL)
		g_free(config->preset_name);
	
	config->preset_name = g_strdup(in_preset_name);
	
	preset_name = config->preset_name;
	
	array = g_strsplit(preset_name,"/",1);
	
	user_target = array[0];
	app_target = array[1];

	
	if (strncmp("user ",user_target,5) == 0)
	{
		usercontext_l = g_list_append(usercontext_l, g_strdup(config->username));
		usercontext_l = g_list_append(usercontext_l, g_strdup(config->groupname));
		usercontext_l = g_list_append(usercontext_l, g_strdup("system"));
	} 
	else if (strncmp("group ",user_target,5) == 0)
	{
		usercontext_l = g_list_append(usercontext_l, g_strdup(config->groupname));
		usercontext_l = g_list_append(usercontext_l, g_strdup("system"));
	} 
	else if (strcmp("system",user_target) == 0)
	{
		usercontext_l = g_list_append(usercontext_l, g_strdup("system"));
	} else {
		g_print("user not allowed!\n");		
		return;
	}
	
	usercontext_l = g_list_append(usercontext_l,g_strdup("factory"));
	
	
	if (strcmp("gtk+",app_target) == 0)
	{
		appcontext_l = g_list_append(appcontext_l, g_strdup("gtk+"));
	} else {
		appcontext_l = g_list_append(appcontext_l, g_strdup(app_target));
		appcontext_l = g_list_append(appcontext_l, g_strdup("gtk+"));
	}

	/* FIXME: lists free'd ?! */
	
	config->magic_usercontext_l = usercontext_l;
	config->magic_appcontext_l = appcontext_l;
	
	gtk_magic_config_server_config_changed(config);
}


