#include <gtk/gtk.h>
#include <gtk/gtkmagic.h>
#include <gtk/gtkprop.h>

typedef struct _GtkShadowPanel GtkShadowPanel;

struct _GtkShadowPanel
{
	GtkMagic *magic;
	GtkProp *prop;
	GtkWidget *from_parent;
	GtkWidget *sensitive_container;
	GtkWidget *but[5];
};

static GQuark shadow_type_quark = 0;

/* prop callbacks */

void
gtk_shadow_panel_prop_data_changed(GtkProp *prop, GPropData *data_new, gpointer user_data)
{
	GtkShadowPanel *panel;
	guint i;
	
	panel = user_data;
	
	g_return_if_fail(data_new->type == G_PROP_DATA_UINT8);
	
	for (i=0;i<5;i++)
		gtk_signal_handler_block_by_data(GTK_OBJECT(panel->but[i]), panel);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(panel->but[data_new->d.uint8_data]), 1);	
	
	for (i=0;i<5;i++)
		gtk_signal_handler_unblock_by_data(GTK_OBJECT(panel->but[i]), panel);
}

void
gtk_shadow_panel_prop_ownership_changed(GtkProp *prop, gboolean ownership, GtkShadowPanel *panel)
{
	gtk_widget_set_sensitive(panel->sensitive_container, ownership);
	gtk_signal_handler_block_by_data(GTK_OBJECT(panel->from_parent), panel);
	
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(panel->from_parent), (!ownership));
	
	gtk_signal_handler_unblock_by_data(GTK_OBJECT(panel->from_parent), panel);
}

/* gui callbacks */

void
gtk_shadow_panel_radio_clicked(GtkToggleButton *button, GtkShadowPanel *panel)
{
	guint shadow_type;
	GPropData *data;

	if (gtk_toggle_button_get_active(button))
	{	
		guint i;
		shadow_type = GPOINTER_TO_UINT(gtk_object_get_data_by_id(GTK_OBJECT(button),shadow_type_quark));
		data = g_prop_data_uint8_new(shadow_type);
		for (i=0;i<5;i++)
			gtk_signal_handler_block_by_data(GTK_OBJECT(panel->but[i]), panel);

			gtk_magic_set_prop(panel->magic, panel->prop->name, data);
		for (i=0;i<5;i++)
			gtk_signal_handler_unblock_by_data(GTK_OBJECT(panel->but[i]), panel);

		g_prop_data_free(data);
	}
	
}

void
gtk_shadow_panel_from_parent_toggled(GtkToggleButton *button, GtkShadowPanel *panel)
{
	gboolean active;
	
	active = gtk_toggle_button_get_active(button);
	if (active)
	{
		gtk_magic_delete_prop(panel->magic, panel->prop->name);
	}
	else
	{
		gtk_magic_set_prop(panel->magic, panel->prop->name, panel->prop->data);
	}
}

void
gtk_shadow_panel_destroy(GtkWidget *widget, GtkShadowPanel *panel)
{
	gtk_magic_unreg_prop(panel->magic, panel->prop);
	gtk_object_destroy(GTK_OBJECT(panel->prop));
	
	g_free(panel);
}

GtkWidget*
gtk_shadow_panel_new(GtkMagic *magic, gchar *widget_name)
{
	gchar *frame_name[5] = { "none","in", "out","etched in", "etched out" };
	gchar *prop_name;
	GtkProp *prop;
	GtkShadowPanel *panel;
	GtkWidget *frame, *vbox, *from_parent, *hbox;
	GSList *radio_group;
	guint i;
	
	if (shadow_type_quark == 0)
		shadow_type_quark = g_quark_from_static_string("shadow type");
	
	prop_name = g_strconcat(widget_name,"::shadow", NULL);
	
	prop = gtk_magic_observe_prop(magic, prop_name, GMAGIC_OBSERVE_DATA|GMAGIC_OBSERVE_OWNERSHIP);
	if (prop == NULL)
	{
		g_print("couldn't observe!\n");
		return NULL;
	}
	
	g_return_val_if_fail(prop->data->type == G_PROP_DATA_UINT8, NULL);
	
	panel = g_new(GtkShadowPanel, 1);
	
	panel->magic = magic;
	panel->prop = prop;
	
	frame = gtk_frame_new("shadow type");

		vbox = gtk_vbox_new(0,0);

		from_parent = gtk_check_button_new_with_label("from parent");
		panel->from_parent = from_parent;
		gtk_widget_show(from_parent);
		gtk_box_pack_start(GTK_BOX(vbox), from_parent, 0,0,0);
		
		hbox = gtk_hbox_new(1,0);
		
		radio_group = NULL;

		for (i=0;i<5;i++)
		{
			GtkWidget *child;
			GtkWidget *but;
			child = gtk_frame_new(NULL);
			gtk_widget_set_usize(GTK_WIDGET(child),48,48);
			gtk_frame_set_shadow_type(GTK_FRAME(child), i);
			gtk_widget_show(child);
			but = gtk_radio_button_new(radio_group);
			gtk_object_set_data_by_id(GTK_OBJECT(but), shadow_type_quark, GUINT_TO_POINTER(i));
			gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(but), 0);
			gtk_button_set_relief(GTK_BUTTON(but),GTK_RELIEF_NONE);
			panel->but[i] = but;
			gtk_container_add(GTK_CONTAINER(but), child);
			gtk_widget_show(but);
			gtk_box_pack_start(GTK_BOX(hbox), but, 0,0,0);
			
			radio_group = gtk_radio_button_group(GTK_RADIO_BUTTON(but));
		}
		
		gtk_widget_show(hbox);
		gtk_box_pack_start(GTK_BOX(vbox),hbox,0,0,0);
		gtk_widget_show(vbox);
		
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	
	panel->sensitive_container = hbox;
	
	if (!prop->ownership)
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(from_parent), TRUE);
		gtk_widget_set_sensitive(hbox, 0);
	}

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(panel->but[prop->data->d.uint8_data]), 1);

	for (i=0;i<5;i++)
		gtk_signal_connect(GTK_OBJECT(panel->but[i]), "toggled", gtk_shadow_panel_radio_clicked, panel);
		
	gtk_signal_connect(GTK_OBJECT(from_parent), "toggled", gtk_shadow_panel_from_parent_toggled, panel);

	gtk_signal_connect(GTK_OBJECT(prop), "data_changed", gtk_shadow_panel_prop_data_changed, panel);
	gtk_signal_connect(GTK_OBJECT(prop), "ownership_changed", gtk_shadow_panel_prop_ownership_changed, panel);
	
	gtk_signal_connect(GTK_OBJECT(vbox), "destroy", gtk_shadow_panel_destroy, panel);

	return frame;
}
