#include "style.h"
#include <gtk/gtk.h>
#include <gdk/gdktypes.h>

typedef struct _GtkStylePanel GtkStylePanel;

struct _GtkStylePanel
{
	GtkMagic *magic;
	GtkProp *prop;
	
	GtkButton *focus_color_button;

	GtkWidget *color_but[5*4];
	GtkWidget *pixmap_but[5];
	GtkWidget *font_entry;
	
	guint focus_color_index;
	guint focus_pixmap_index;
		
	GtkColorSelectionDialog *color_sel;
	GtkFontSelectionDialog *font_sel;
	GtkFileSelection *file_sel;
	
	GtkWidget *from_parent;
	GtkWidget *sensitive_container;

	GtkRcStyle *rc_style;
};

static GQuark index_quark;

/* prop -> gui */

void
gtk_style_panel_update(GtkStylePanel *panel, GPropData *propdata)
{
	GtkRcStyle *rc_style = NULL, *tmprc;
	GtkStyle *style;
	GdkFont *font;
	guint i;

	rc_style = gtk_rc_style_new_from_prop_data(propdata);
	
	if (rc_style == NULL)
	{
		g_print("propdata isn't rc_style data!\n");
		return;
	}
		
	if (panel->rc_style != NULL)
		gtk_rc_style_unref(panel->rc_style);
	
	panel->rc_style = rc_style;
	
	style = gtk_style_new();
	font = gdk_font_load(rc_style->font_name);
	if (font != NULL)
	{
		if (style->font != NULL)
			gdk_font_unref(style->font);
		style->font = font;
	}
	gtk_widget_set_style(panel->font_entry, style);
	gtk_style_unref(style);

	
	for (i=0;i<5;i++)
	{
		gushort r,g,b;
		guint j;
		
		r = rc_style->fg[i].red;
		g = rc_style->fg[i].green;
		b = rc_style->fg[i].blue;
		
		style = gtk_style_new();
		for(j=0;j<5;j++)
		{
			style->bg[j].red = r;
			style->bg[j].green = g;
			style->bg[j].blue = b;	
		}
		gtk_widget_set_style(panel->color_but[i*4+0], style);
		gtk_style_unref(style);

		
		r = rc_style->bg[i].red;
		g = rc_style->bg[i].green;
		b = rc_style->bg[i].blue;

		style = gtk_style_new();
		for(j=0;j<5;j++)
		{
			style->bg[j].red = r;
			style->bg[j].green = g;
			style->bg[j].blue = b;
		}
		gtk_widget_set_style(panel->color_but[i*4+1], style);
		gtk_style_unref(style);

	

		r = rc_style->text[i].red;
		g = rc_style->text[i].green;
		b = rc_style->text[i].blue;

		style = gtk_style_new();
		for(j=0;j<5;j++)
		{
			style->bg[j].red = r;
			style->bg[j].green = g;
			style->bg[j].blue = b;
		}
		gtk_widget_set_style(panel->color_but[i*4+2], style);
		gtk_style_unref(style);

	
		
		r = rc_style->base[i].red;
		g = rc_style->base[i].green;
		b = rc_style->base[i].blue;

		style = gtk_style_new();
		for(j=0;j<5;j++)
		{
			style->bg[j].red = r;
			style->bg[j].green = g;
			style->bg[j].blue = b;
		}
		gtk_widget_set_style(panel->color_but[i*4+3], style);
		gtk_style_unref(style);

	
/*		style = gtk_style_new();
		tmprc = gtk_rc_style_new();
		for(j=0;j<5;j++)
		{
			tmprc->bg_pixmap_name[j] = g_strdup(rc_style->bg_pixmap_name[i]);
		}
		style->rc_style = tmprc;
		gtk_widget_set_style(panel->pixmap_but[i], style);
		gtk_style_unref(style); */

	}
}

/* gui -> prop */

void
gtk_style_panel_set_prop(GtkStylePanel *panel)
{
	GPropData *propdata;
	
	g_return_if_fail(panel->rc_style != NULL);
	
	propdata = gtk_rc_style_to_prop_data(panel->rc_style);
	
	gtk_magic_set_prop(panel->magic, panel->prop->name, propdata);
	
	g_prop_data_free(propdata);
}


void
gtk_style_panel_focus_color_changed(GtkStylePanel *panel, GdkColor *color)
{
	guint index;
	
	index = panel->focus_color_index;
	
	panel->rc_style->fg[((index%4)*5)+(index/4)].red = color->red;
	panel->rc_style->fg[((index%4)*5)+(index/4)].green = color->green;
	panel->rc_style->fg[((index%4)*5)+(index/4)].blue = color->blue;
		
	gtk_style_panel_set_prop(panel);
}

/* --- gui callbacks --- */

/*
void
gtk_style_panel_file_ok(GtkButton *ok_button, gpointer user_data)
{
	GtkStylePanel *panel;
	gchar *pixmap_name;
	
	panel = user_data;
	pixmap_name = gtk_file_selection_get_filename(GTK_FILE_SELECTION(panel->file_sel));

	if (panel->rc_style->bg_pixmap_name[panel->focus_pixmap_index] != NULL)
		g_free(panel->rc_style->bg_pixmap_name[panel->focus_pixmap_index]);

	panel->rc_style->bg_pixmap_name[panel->focus_pixmap_index] = g_strdup(pixmap_name);
	
	gtk_style_panel_set_prop(panel);
	
	gtk_widget_destroy(GTK_WIDGET(panel->file_sel));
}
*/

void
gtk_style_panel_font_ok(GtkButton *ok_button, gpointer user_data)
{
	GtkStylePanel *panel;
	gchar *fontname;
	
	panel = user_data;
	fontname = gtk_font_selection_get_font_name(GTK_FONT_SELECTION(panel->font_sel->fontsel));
	
	g_return_if_fail(panel->rc_style != NULL);
	
	if (panel->rc_style->font_name != NULL)
		g_free(panel->rc_style->font_name);
	
	panel->rc_style->font_name = g_strdup(fontname);
	
	gtk_style_panel_set_prop(panel);
	
	gtk_widget_destroy(GTK_WIDGET(panel->font_sel));
}

/*
void
gtk_style_panel_file_cancel(GtkButton *cancel_button, gpointer user_data)
{
	GtkStylePanel *panel;
	
	panel = user_data;
	gtk_widget_destroy(GTK_WIDGET(panel->file_sel));
}
*/

void
gtk_style_panel_font_cancel(GtkButton *cancel_button, gpointer user_data)
{
	GtkStylePanel *panel;
	
	panel = user_data;
	gtk_widget_destroy(GTK_WIDGET(panel->font_sel));
}

/*
void
gtk_style_panel_file_destroy(GtkWidget *dialog, gpointer user_data)
{
	GtkStylePanel *panel;
	panel = user_data;

	panel->file_sel = NULL;
}
*/

void
gtk_style_panel_font_dialog_destroy(GtkWidget *dialog, gpointer user_data)
{
	GtkStylePanel *panel;
	panel = user_data;
	panel->font_sel = NULL;
}

void
gtk_style_panel_color_ok(GtkButton *ok_button, gpointer user_data)
{
	GtkStylePanel *panel;
	/* close dialog */
	
	panel = user_data;
	gtk_widget_destroy(GTK_WIDGET(panel->color_sel));
}

void
gtk_style_panel_color_cancel(GtkButton *cancel_button, gpointer user_data)
{
	GtkStylePanel *panel;
	/* close dialog and reset button colors */
	
	panel = user_data;
	gtk_widget_destroy(GTK_WIDGET(panel->color_sel));
}

void
gtk_style_panel_color_dialog_destroy(GtkWidget *dialog_widget, gpointer user_data)
{
	GtkStylePanel *panel;
	
	panel = user_data;
	panel->color_sel = NULL;
}

void
gtk_style_panel_color_changed(GtkColorSelection *color_selection, gpointer user_data)
{
	GtkStylePanel *panel;
	gdouble double_ar[3];
	GdkColor color;	
	
	panel = user_data;
	
	gtk_color_selection_get_color(color_selection,double_ar);
	
	color.red = (gushort)(double_ar[0]*65535.0);
	color.green = (gushort)(double_ar[1]*65535.0);
	color.blue = (gushort)(double_ar[2]*65535.0);
	
	gtk_style_panel_focus_color_changed(panel, &color);
}

/*
void
gtk_style_panel_pixmap_pressed(GtkButton *pixmap_button, gpointer user_data)
{
	GtkStylePanel *panel;
	
	panel = user_data;

	if (panel->file_sel == NULL)
	{
		GtkFileSelection *file_sel;
		file_sel = GTK_FILE_SELECTION(gtk_file_selection_new(panel->prop->name));
		gtk_signal_connect(GTK_OBJECT(file_sel->ok_button),"pressed",gtk_style_panel_file_ok, panel);
		gtk_signal_connect(GTK_OBJECT(file_sel->cancel_button),"pressed",gtk_style_panel_file_cancel, panel);
		gtk_widget_show(GTK_WIDGET(file_sel));
		gtk_signal_connect(GTK_OBJECT(file_sel),"destroy",gtk_style_panel_file_destroy, panel);
		panel->file_sel = file_sel;
	}
	
	panel->focus_pixmap_index = GPOINTER_TO_UINT(gtk_object_get_data_by_id(GTK_OBJECT(pixmap_button), index_quark));
}
*/

void
gtk_style_panel_color_realized(GtkWidget *widget, gpointer user_data)
{
  	gdouble *double_ar;
	GtkColorSelection *colorsel;
	colorsel = GTK_COLOR_SELECTION(widget);
	double_ar = user_data;
	
	gtk_color_selection_set_color(colorsel, double_ar);
	g_free(double_ar);
	gtk_signal_disconnect_by_data(GTK_OBJECT(widget), double_ar);
}

void
gtk_style_panel_color_pressed(GtkButton *color_button, gpointer user_data)
{
	GtkStylePanel *panel;
	GtkStyle *but_style;
	gdouble *double_ar;
	gushort red,green,blue;

	panel = user_data;
	
	if (panel->color_sel == NULL)
	{
		GtkWidget *widget;
		GtkColorSelectionDialog *dialog;
		widget = gtk_color_selection_dialog_new(panel->prop->name);
		dialog = GTK_COLOR_SELECTION_DIALOG(widget);
		gtk_signal_connect(GTK_OBJECT(dialog->ok_button),"pressed", gtk_style_panel_color_ok, panel);
		gtk_signal_connect(GTK_OBJECT(dialog->cancel_button),"pressed", gtk_style_panel_color_cancel, panel);
		gtk_signal_connect(GTK_OBJECT(dialog),"destroy", gtk_style_panel_color_dialog_destroy, panel);
		gtk_signal_connect(GTK_OBJECT(dialog->colorsel), "color_changed", gtk_style_panel_color_changed, panel);
		panel->color_sel = dialog;
		gtk_widget_show(widget);
		gtk_main_iteration();
	}
	
	double_ar = g_malloc(sizeof(gdouble)*3);
	
	panel->focus_color_index = GPOINTER_TO_UINT(gtk_object_get_data_by_id(GTK_OBJECT(color_button), index_quark));
	
	but_style = gtk_widget_get_style(GTK_WIDGET(color_button));
	red = but_style->bg[0].red;
	green = but_style->bg[0].green;
	blue = but_style->bg[0].blue;
	
	double_ar[0] = (gdouble)red/65535.0;
	double_ar[1] = (gdouble)green/65535.0;
	double_ar[2] = (gdouble)blue/65535.0;

	if (!GTK_WIDGET_REALIZED(panel->color_sel->colorsel))
		gtk_signal_connect(GTK_OBJECT(panel->color_sel->colorsel), "realized", gtk_style_panel_color_realized, double_ar);
	else
	{
		gtk_color_selection_set_color(GTK_COLOR_SELECTION(panel->color_sel->colorsel), double_ar);
		g_free(double_ar);
	}
}

void
gtk_style_panel_font_pressed(GtkButton *font_button, gpointer user_data)
{
	GtkStylePanel *panel;
	
	panel = user_data;
	
	if (panel->font_sel == NULL)
	{
		GtkWidget *widget;
		GtkFontSelectionDialog *dialog;
		widget = gtk_font_selection_dialog_new(panel->prop->name);
		dialog = GTK_FONT_SELECTION_DIALOG(widget);
		gtk_signal_connect(GTK_OBJECT(dialog->ok_button),"pressed", gtk_style_panel_font_ok, panel);
		gtk_signal_connect(GTK_OBJECT(dialog->cancel_button), "pressed", gtk_style_panel_font_cancel, panel);
		gtk_signal_connect(GTK_OBJECT(dialog),"destroy", gtk_style_panel_font_dialog_destroy, panel);
		panel->font_sel = dialog;
		gtk_widget_show(widget);
		if (panel->rc_style->font_name != NULL)
			gtk_font_selection_set_font_name(GTK_FONT_SELECTION(dialog->fontsel), panel->rc_style->font_name);
	}
}


void
gtk_style_panel_prop_data_changed(GtkProp *prop, GPropData *propdata_new, gpointer user_data)
{
	GtkStylePanel *panel;
	panel = user_data;
	gtk_style_panel_update(panel, propdata_new);
}

void
gtk_style_panel_prop_ownership_changed(GtkProp *prop, gboolean ownership, gpointer user_data)
{
	GtkStylePanel *panel;
	panel = user_data;
	gtk_widget_set_sensitive(panel->sensitive_container, ownership);
	if (panel->color_sel != NULL)
		gtk_widget_set_sensitive(GTK_WIDGET(panel->color_sel), ownership);
	if (panel->font_sel != NULL)
		gtk_widget_set_sensitive(GTK_WIDGET(panel->font_sel), ownership);

	gtk_signal_handler_block_by_data(GTK_OBJECT(panel->from_parent), panel);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(panel->from_parent), (!ownership));
	gtk_signal_handler_unblock_by_data(GTK_OBJECT(panel->from_parent), panel);
}

void
gtk_style_panel_from_parent_button_toggled(GtkToggleButton *button, gpointer user_data)
{
	GtkStylePanel *panel;
	gboolean active;

	panel = user_data;
	
	active = gtk_toggle_button_get_active(button);
	
	if (active)
	{
		/* from parent */
		gtk_magic_delete_prop(panel->magic, panel->prop->name);
	}
	else
	{
		/* ownership */
		
		gtk_magic_set_prop(panel->magic, panel->prop->name, panel->prop->data);
	}
}

void
gtk_style_panel_destroy(GtkObject *object, GtkStylePanel *panel)
{
	if (panel->prop != NULL)
	{
		gtk_magic_unreg_prop(panel->magic, panel->prop);
		gtk_object_destroy(GTK_OBJECT(panel->prop));
	}
	if (panel->color_sel != NULL)
		gtk_widget_destroy(GTK_WIDGET(panel->color_sel));
	if (panel->font_sel != NULL)
		gtk_widget_destroy(GTK_WIDGET(panel->font_sel));
	if (panel->file_sel != NULL)
		gtk_widget_destroy(GTK_WIDGET(panel->file_sel));
	if (panel->rc_style != NULL)
		gtk_rc_style_unref(panel->rc_style);
	g_free(panel);
}

GtkWidget*
gtk_style_panel_new(GtkMagic *magic, gchar *widget_name)
{
	GtkStylePanel *panel;
	GtkProp *prop=NULL;
	GtkWidget *vbox1, *from_parent, *vbox2, *nb, *label, *hbox, *but, *font_entry;
	gint j,i;
	gchar *prop_name;

	/* global stuff */
	
	if (index_quark == 0)
		index_quark = g_quark_from_static_string("index");
		
	/* object construction */

	prop_name = g_strconcat(widget_name,"::rcstyle", NULL);
	
	prop = gtk_magic_observe_prop(magic, prop_name, GMAGIC_OBSERVE_DATA|GMAGIC_OBSERVE_OWNERSHIP);
	g_free(prop_name);
	if (prop == NULL)
	{
		g_print("couldn't observe!\n");
		return NULL;
	}

	g_return_val_if_fail(prop->data->type == G_PROP_DATA_LIST, NULL);

	panel = g_new(GtkStylePanel, 1);

	panel->magic = magic;
	panel->prop = prop;	
	panel->focus_color_index = 0;
	panel->focus_pixmap_index = 0;
	panel->color_sel = NULL;
	panel->font_sel = NULL;
	panel->file_sel = NULL;
	panel->rc_style = NULL;
		
	vbox1 = gtk_vbox_new(0,0);

	from_parent = gtk_check_button_new_with_label("from parent");
	panel->from_parent = from_parent;
	gtk_widget_show(from_parent);
	gtk_box_pack_start(GTK_BOX(vbox1), from_parent, 0,0,0);


	vbox2 = gtk_vbox_new(0,0);
	
		nb = gtk_notebook_new();
		for(j=0;j<5;j++)
		{
			GtkWidget *table;
			gchar *nb_label[5] = {"Normal","Active","Prelight","Selected","Insensitve"};
			
			table = gtk_table_new(8,2,0);
			for(i=0;i<5;i++)
			{

				GtkWidget *label, *but;
				gchar *color_label[5] = {"FG","BG","Text","Base","BG Pixmap"};
				
				label = gtk_label_new(color_label[i]);
				gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);
				gtk_widget_show(label);
				but = gtk_button_new();
			
				if (i < 4)
				{
					panel->color_but[(j*4)+i] = but;
					gtk_object_set_data_by_id(GTK_OBJECT(but), index_quark, GUINT_TO_POINTER((j*4)+i));
					gtk_signal_connect(GTK_OBJECT(but), "pressed", gtk_style_panel_color_pressed, panel);
				}
				else
				{
/*					panel->pixmap_but[j] = but;
					gtk_object_set_data_by_id(GTK_OBJECT(but), index_quark, GUINT_TO_POINTER(j));
					gtk_signal_connect(GTK_OBJECT(but), "pressed", gtk_style_panel_pixmap_pressed, panel); */
				}
			
				if (i < 4)
				{	
				gtk_widget_show(but);
		
				gtk_table_attach(GTK_TABLE(table), label, 0,1,i,i+1, 0, 0,0,0);
				gtk_table_attach(GTK_TABLE(table), but, 1,2,i,i+1, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);
				}
			}
			gtk_widget_show(table);
			
			label = gtk_label_new(nb_label[j]);
			gtk_widget_show(label);
			gtk_notebook_append_page(GTK_NOTEBOOK(nb), table, label);
		}
		gtk_widget_show(nb);
		gtk_box_pack_start(GTK_BOX(vbox2), nb, 1,1,0);
		
		hbox = gtk_hbox_new(0,0);
			label = gtk_label_new("Font:");
			gtk_widget_show(label);
			gtk_box_pack_start(GTK_BOX(hbox), label, 0,0,0);
			font_entry = gtk_entry_new();
			gtk_entry_set_text(GTK_ENTRY(font_entry), "Sample text");
			panel->font_entry = font_entry;
			gtk_widget_show(font_entry);
			gtk_box_pack_start(GTK_BOX(hbox), font_entry, 1,1,0);
			but = gtk_button_new_with_label("Choose Font...");
			gtk_signal_connect(GTK_OBJECT(but), "pressed", gtk_style_panel_font_pressed, panel);
			gtk_widget_show(but);
			gtk_box_pack_start(GTK_BOX(hbox), but, 0,0,0);
		gtk_widget_show(hbox);
		gtk_box_pack_start(GTK_BOX(vbox2), hbox, 0,0,0);
	
	gtk_widget_show(vbox2);	
	gtk_box_pack_start(GTK_BOX(vbox1), vbox2, 1,1,0);
	panel->sensitive_container = vbox2;

	gtk_signal_connect(GTK_OBJECT(prop), "data_changed", gtk_style_panel_prop_data_changed, panel);
	
	gtk_signal_connect(GTK_OBJECT(prop), "ownership_changed", gtk_style_panel_prop_ownership_changed, panel);
	if (!prop->ownership)
	{
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(from_parent), TRUE);
		gtk_widget_set_sensitive(panel->sensitive_container, FALSE);
	}

	gtk_signal_connect(GTK_OBJECT(from_parent), "toggled", gtk_style_panel_from_parent_button_toggled, panel); 

	gtk_style_panel_update(panel,panel->prop->data);
	
	gtk_signal_connect(GTK_OBJECT(vbox1), "destroy", gtk_style_panel_destroy, panel);
	
	gtk_widget_show(vbox1);
	
	return vbox1;			
}


